;========================================================
;
; Copyright (C) 1987  Metacomco plc. All rights reserved
;
; Standard startup code for Lattice C V3.04 on the Atari
;
;========================================================
;
; Equates
;
space    EQU      $20
tab      EQU      $09               ; tab character
newline  EQU      $0A               ; newline
;
nam_siz  EQU      32                ; max size of name
min_mem  EQU      $800              ; 2k default memory allocation
min_stk  EQU      $400              ; 1k minimum stack
sys_mem  EQU      $1000             ; 4k left for TOS
;
gem_shrk EQU      $4A               ; TOS shrink memory function
gem_term EQU      $4C               ; Terminate process function
;
; Text section
;
         TEXT
;
         XDEF      CXINIT
         XDEF      XCEXIT
;
         XREF     _main
         XREF     _STACK
         XREF     _MNEED
         XREF     rbrk
;
; Enter here with address of base page on stack above return address.
;
CXINIT:
         move.l   4(a7),a1          ; get address of base page in a1
         move.l   a1,_pbase         ; save address of base page
         move.l   $2C(a1),a5        ; save packed enviroment array
         lea      $80(a1),a2        ; get address of command tail
         clr.l    d2
         move.b   (a2)+,d2          ; get length of string
         move.l   a2,a3             ; save address of command tail
         move.l   a2,a4             ;
         subq.w   #1,d2             ; for dbra
;
scan:
         cmpi.b   #newline,(a2)     ; end of line ?
         beq      end_line
         cmpi.b   #space,(a2)       ; space ?
         beq.s    scan1
         cmpi.b   #tab,(a2)         ; tab ?
         bne.s    scan2             ; no, try others
;
scan1    addq.l   #1,a2
         subq.w   #1,d2
         bpl.s    scan
         bra      end_line          ; end of line
;
scan2    cmpi.b   #'<',(a2)         ; start of input file name ?
         beq.s    inp_nam
         cmpi.b   #'>',(a2)         ; start of output file name ?
         beq.s    out_nam
         cmpi.b   #'=',(a2)         ; stack size specification ?
         bne.s    scan3
         bsr.s    get_num
         cmp.l    #min_stk,d0
         bgt.s    stk_1
         move.l   #min_stk,d0
stk_1    move.l   d0,_STACK         ; set memory allocate size
         bra.s    scan
;
scan3    cmp.b    #'%',(a2)         ; workspace specification ?
         bne      end_line
         bsr.s    get_num
         cmp.l    #min_mem,d0
         bgt.s    wrk_1
         move.l   #min_mem,d0
wrk_1    move.l   d0,_MNEED         ; set memory allocate size
         bra.s    scan
;
; get number function
;
get_num:
         addq.l   #1,a2             ; move past '=' or '%'
         moveq    #0,d0             ; clear accumulator
;
get_num1 moveq    #0,d1
         move.b   (a2),d1           ; get digit
         cmp.w    #$39,d1           ; check for numeric
         bgt.s    get_num2
         sub.w    #$30,d1
         blt.s    get_num2
         asl.l    #1,d0             ; x2
         move.l   d0,-(a7)
         asl.l    #2,d0             ; x8
         add.l    (a7)+,d0          ; x10
         add.l    d1,d0             ; add in new digit
         addq.l   #1,a2             ; move to next digit
         subq.w   #1,d2
         bmi      error
         bra.s    get_num1
;
get_num2 tst.l    d0
         beq      error             ; if specification error
         rts
;
; redirection       
;        
out_nam:
         lea      _oname,a0
         bra      get_nam
;        
inp_nam:
         lea      _iname,a0
get_nam:
         moveq    #nam_siz-1,d1     ; set size of name field
         addq.l   #1,a2             ; get past "<" or ">"
get_nam1
         subq.w   #1,d2
         bmi.s    end_line
         cmpi.b   #space,(a2)       ; end of name ?
         beq      scan
         cmpi.b   #tab,(a2)
         beq      scan
         cmpi.b   #newline,(a2)
         beq      scan
         move.b   (a2)+,(a0)+
         dbf      d1,get_nam1
;
error:
         moveq    #-1,d0            ; set error
         bra      abort             ; exit
;
end_line:
;
; compress command line
;
         tst.w    d2
         bmi.s    end2              ; if nothing to compress
;
end1     move.b   (a2)+,(a3)+
         dbf      d2,end1
;
end2     clr.b    (a3)+
;        
; allocate space for memory allocation and stack
;
mem:
         move.l   $18(a1),d2        ; base of BSS
         add.l    $1C(a1),d2        ; size of BSS - d2 = base of free memory
         move.l   a7,d0             ; d0 = top of memory
         sub.l    _STACK,d0         ; allow for declared stack size
         sub.l    d2,d0             ; d0 = size of free memory
         move.l   _MNEED,d3         ; d3 = requested memory
         bne.s    mem1              ; if zero
         move.l   d0,d3
         sub.l    #sys_mem,d3       ; then leave some for TOS
         bra.s    mem3
;
mem1     bgt.s    mem2              ; if > 0 get that much
         add.l    d0,d3             ; if < 0 leave that much for TOS
;
mem2     cmp.l    #min_mem,d3       ; make sure we get at least the minimum
         bgt.s    mem3
         move.l   #min_mem,d3       ; set minimum memory size
;
mem3     move.l   d2,d0             ; d0 = base of free memory
         sub.l    a1,d0             ; d0 = size of program
         add.l    d3,d0             ; add heap requirements
         add.l    _STACK,d0         ; add stack requirements
         move.l   d0,-(a7)          ; length of retained memory
         move.l   a1,-(a7)          ; base of retained memory
         clr.w    -(a7)             ; filler
         move.w   #gem_shrk,-(a7)   ; function code
         trap     #1                ; Memory now shrunk
         tst.l    d0                ; check it worked
         bne.s    abort
;
mem4     move.l   d2,a7             ; set up stack
         add.l    d3,a7             ; add heap size
         move.l   a7,_base          ; set base of stack
         add.l    _STACK,a7         ; add stack size for new sp
         move.l   a7,_top           ; set top of stack
         move.l   d2,_mbase         ; set base of heap
         move.l   d2,_mnext         ; set heap next pointer
         move.l   d3,_msize         ; set size of heap
         jsr      rbrk
;
         move.l   a5,-(a7)          ; move packed enviroment array to stack
         move.l   a4,-(a7)          ; move command line address to stack
;
         lea.l    dstart+$8000,a5   ; set up A5 and A6 for position...
         move.l   a5,a6             ; ...independent code
;
         jsr      _main
;
         moveq    #0,d0
exit:
         tst.l    _ONEXIT
         beq.s    abort
         move.l   d0,-(a7)
         move.l   _ONEXIT,a0
         jsr      (a0)
         addq     #4,a7
abort:
         move.w   d0,-(a7)          ; set return code
         move.w   #gem_term,-(a7)   ; GEMDOS terminate function
;
abort1   trap     #1                ; should not return
         bra.s    abort1
;
XCEXIT:
         move.l   4(a7),d0          ; save return status
         bra.s    exit
;
; Data section
;
         DATA
;
         XDEF     _iname,_oname
         XDEF     _base,_top
         XDEF     _mbase,_mnext,_msize
         XDEF     _pbase,environ
         XDEF     _OSERR,_FPERR,_ONEXIT
;
dstart:
_iname   DS.B     nam_siz           ; input file name
_oname   DS.B     nam_siz           ; output file name
_base    DC.L     0                 ; base of stack
_top     DC.L     0                 ; top of stack
_mbase   DC.L     0                 ; base of heap
_mnext   DC.L     0                 ; next available heap location
_msize   DC.L     0                 ; size of heap
_pbase   DC.L     0                 ; pointer to base page
environ  DC.L     0                 ; pointer to enviroment strings
_OSERR   DC.L     0                 ; system error
_FPERR   DC.L     0                 ;
_ONEXIT  DC.L     0                 ; exit trap
;     
         END
