
#include <dos.h>
#include <stdio.h>
#include <ctype.h>
#include <ios1.h>
#include <osbind.h>

#define MAXARG 32              /* maximum command line arguments */

#ifndef TINY
extern int _fmode;
extern char _iname[], _oname[];
#endif

extern char **environ;

int argc;                       /* arg count */
char *argv[MAXARG];             /* arg pointers */

/**
*
* name         _main - process command line, open files, and call "main"
*
* synopsis     _main(line, env);
*              char *line;     pointer to command tail
*              char *env;      pointer to environment string
*
* description   This function performs the standard pre-processing for
*               the main module of a C program.  It accepts a command
*               tail of the form:
*
*                       arg1 arg2 ...
*
*               and builds a list of pointers to each argument.  The first
*               pointer is to a dummy program name 'CPROG'.  The standard
*               I/O files are also opened, using file names that were set
*               up by the startup module.
*
**/
void
_main(line, env)
char *line, *env;
{
  char c;

#ifndef TINY
  int fh0, fh1;
  int x;
#endif

/*
*
* Build argument pointer list
*
*/
  argv[0] = "CPROG";
  for (argc = 1; argc < MAXARG; )
  {
    while (isspace(*line)) line++;
    if (*line == '\0') break;
    argv[argc++] = line;
    while ((*line != '\0') && (isspace(*line) == 0)) line++;
    c = *line;
    *line++ = '\0';
    if (c == '\0') break;
  }

/*
*
* Open standard files
*
*/
#ifndef TINY
  x = (_fmode) ? 0 : _IOXLAT;
  stdin->_file = 0;
  stdin->_flag = _IOREAD | x;
  stdout->_file = 1;
  stdout->_flag = _IOWRT | _IONBF | x;
  stderr->_file = 0xFFFF;
  stderr->_flag = _IORW | _IONBF | x;
  stdaux->_file = 2;
  stdaux->_flag = _IORW | x;
  stdprt->_file = 3;
  stdprt->_flag = _IOWRT | x;

  if (_iname[0] != '\0')
  {
    if ((fh0 = Fopen(_iname, 0)) < 0)
    {
      fputs("Can't open stdin file\n", stderr);
      exit(1);
    }
    Fforce(0, fh0);
  }
  if (_oname[0] != '\0')
  {
    if (_oname[0] == '>') fh1 = Fopen(&_oname[1], 0);
    else fh1 = Fcreate(_oname, 0);
    if (fh1 < 0)
    {
      fputs("Can't open stdout file\n", stderr);
      exit(1);
    }
    if (_oname[0] == '>') Fseek(0, fh1, 2);
    Fforce(1, fh1);
  }
#endif

/*
*
* Unpack the enviroment
*
*/
  envunpk(env);

/*
*
* Call user's main program
*
*/
  main(argc,argv,environ);                /* call main function */
#ifndef TINY
  exit(0);
#else
  _exit(0);
#endif
}

