
/* A8VIEW, simple viewer for Atari XL/XE textfiles.
 * Companion program for EmuXL, the Atari XL/XE emulator
 * (c) 1998 Draco
 */

#include <stdio.h>
#include <string.h>
#include <atarierr.h>
#include <osbind.h>
#include <mintbind.h>
#include "dostruct.h"

short dom;

long findfile(char * name)
{
	DTA dta, *olddta, *newdta;
	XATTR atr, *atrptr;
	long len = 0, ok;

	if (dom) {
		atrptr = &atr;
		if ((ok = Fxattr(0, name, (void *)atrptr)) == 0)
			len = atrptr->size;
	} else {
		olddta = (DTA *)Fgetdta();
		newdta = &dta;
		(void)Fsetdta((void *)newdta);
		if ((ok = Fsfirst(name, 0)) == 0)
			len = newdta->length;
		(void)Fsetdta((void *)olddta);
	}

	return len;
}

/* On dir == 1 converts Unix -> XL, reverse otherwise */

#define MAX_TAB 2
#define TAB	0x7f
#define	EOL	0x9b

void convert(short dir, unsigned char * buf, long len)
{
	long i;
	short x;
	unsigned char xl[MAX_TAB] =
		{ TAB, EOL };
	unsigned char unix[MAX_TAB] =
		{ 0x09, 0x0a };

	if (dir) {
		for ( i = 0; i < len; i++) {
			for (x = 0; x < MAX_TAB; x++) {
				if (buf[i] == unix[x])
					buf[i] = xl[x];
			}
		}
	} else {
		for ( i = 0; i < len; i++) {
			for (x = 0; x < MAX_TAB; x++) {
				if (buf[i] == xl[x])
					buf[i] = unix[x];
			}
			buf[i] &= 0x7f;
		}
	}

}
	
int main(short argc, char * argv[])
{
	short dir = 0, handle, r = 0;
	char infile[512];
	char *buf;
	long len, actual_len;

	if (argc < 2) {
		puts("XL <-> Unix textfile converter\nUsage: [-x] infile\n");
		return E_OK;
	}

	dom = (Pdomain(1) == 0) ? 1 : 0;
	dir = (strcmp(argv[1], "-x") == 0) ? 1 : 0;
	strcpy(infile, argv[1+dir]);

	len = findfile(infile);
	if (len == 0)
		return ERANGE;

	buf = (char *)Mxalloc(len + 1, 3);
	if (buf == NULL)
		return ENSMEM;

	buf[len] = '\0';

	if ((handle = Fopen(infile, 0)) < 0) {
		r = handle;
		goto exit;
	}

	actual_len = Fread(handle, len, buf);
	(void)Fclose(handle);

	if (actual_len <= 0) {
		r = actual_len;
		goto exit;
	}

	if (actual_len < len)
		len = actual_len;

	convert(dir, buf, len);
	puts(buf);

exit:	Mfree(buf);
	return r;
}
